@@ -1,3 +1,3 @@
 dofile(minetest.get_modpath("testentities").."/visuals.lua")
+dofile(minetest.get_modpath("testentities").."/selectionbox.lua")
 dofile(minetest.get_modpath("testentities").."/armor.lua")
-dofile(minetest.get_modpath("testentities").."/callbacks.lua")
