@@ -1,23 +1,28 @@
 local pi = math.pi
-local player_in_bed = 0
 local is_sp = minetest.is_singleplayer()
 local enable_respawn = minetest.settings:get_bool("enable_bed_respawn")
 if enable_respawn == nil then
 	enable_respawn = true
 end
 
+-- support for MT game translation.
+local S = beds.get_translator
+
 -- Helper functions
 
 local function get_look_yaw(pos)
-	local n = minetest.get_node(pos)
-	if n.param2 == 1 then
-		return pi / 2, n.param2
-	elseif n.param2 == 3 then
-		return -pi / 2, n.param2
-	elseif n.param2 == 0 then
-		return pi, n.param2
+	local rotation = minetest.get_node(pos).param2
+	if rotation > 3 then
+		rotation = rotation % 4 -- Mask colorfacedir values
+	end
+	if rotation == 1 then
+		return pi / 2, rotation
+	elseif rotation == 3 then
+		return -pi / 2, rotation
+	elseif rotation == 0 then
+		return pi, rotation
 	else
-		return 0, n.param2
+		return 0, rotation
 	end
 end
 
@@ -55,61 +60,105 @@ local function lay_down(player, pos, bed_pos, state, skip)
 
 	-- stand up
 	if state ~= nil and not state then
-		local p = beds.pos[name] or nil
-		if beds.player[name] ~= nil then
-			beds.player[name] = nil
-			player_in_bed = player_in_bed - 1
+		if not beds.player[name] then
+			-- player not in bed, do nothing
+			return false
 		end
+		beds.bed_position[name] = nil
 		-- skip here to prevent sending player specific changes (used for leaving players)
 		if skip then
 			return
 		end
-		if p then
-			player:setpos(p)
-		end
+		player:set_pos(beds.pos[name])
 
 		-- physics, eye_offset, etc
+		local physics_override = beds.player[name].physics_override
+		beds.player[name] = nil
+		player:set_physics_override({
+			speed = physics_override.speed,
+			jump = physics_override.jump,
+			gravity = physics_override.gravity
+		})
 		player:set_eye_offset({x = 0, y = 0, z = 0}, {x = 0, y = 0, z = 0})
 		player:set_look_horizontal(math.random(1, 180) / 100)
-		default.player_attached[name] = false
-		player:set_physics_override(1, 1, 1)
+		player_api.player_attached[name] = false
 		hud_flags.wielditem = true
-		default.player_set_animation(player, "stand" , 30)
+		player_api.set_animation(player, "stand" , 30)
 
 	-- lay down
 	else
-		beds.player[name] = 1
-		beds.pos[name] = pos
-		player_in_bed = player_in_bed + 1
 
-		-- physics, eye_offset, etc
-		player:set_eye_offset({x = 0, y = -13, z = 0}, {x = 0, y = 0, z = 0})
+		-- Check if bed is occupied
+		for _, other_pos in pairs(beds.bed_position) do
+			if vector.distance(bed_pos, other_pos) < 0.1 then
+				minetest.chat_send_player(name, S("This bed is already occupied!"))
+				return false
+			end
+		end
+
+		-- Check if player is moving
+		if vector.length(player:get_velocity()) > 0.001 then
+			minetest.chat_send_player(name, S("You have to stop moving before going to bed!"))
+			return false
+		end
+
+		-- Check if player is attached to an object
+		if player:get_attach() then
+			return false
+		end
+
+		if beds.player[name] then
+			-- player already in bed, do nothing
+			return false
+		end
+
+		beds.pos[name] = pos
+		beds.bed_position[name] = bed_pos
+		beds.player[name] = {physics_override = player:get_physics_override()}
+
 		local yaw, param2 = get_look_yaw(bed_pos)
 		player:set_look_horizontal(yaw)
 		local dir = minetest.facedir_to_dir(param2)
-		local p = {x = bed_pos.x + dir.x / 2, y = bed_pos.y, z = bed_pos.z + dir.z / 2}
-		player:set_physics_override(0, 0, 0)
-		player:setpos(p)
-		default.player_attached[name] = true
+		-- p.y is just above the nodebox height of the 'Simple Bed' (the highest bed),
+		-- to avoid sinking down through the bed.
+		local p = {
+			x = bed_pos.x + dir.x / 2,
+			y = bed_pos.y + 0.07,
+			z = bed_pos.z + dir.z / 2
+		}
+		player:set_physics_override({speed = 0, jump = 0, gravity = 0})
+		player:set_pos(p)
+		player_api.player_attached[name] = true
 		hud_flags.wielditem = false
-		default.player_set_animation(player, "lay" , 0)
+		player_api.set_animation(player, "lay" , 0)
 	end
 
 	player:hud_set_flags(hud_flags)
 end
 
+local function get_player_in_bed_count()
+	local c = 0
+	for _, _ in pairs(beds.player) do
+		c = c + 1
+	end
+	return c
+end
+
 local function update_formspecs(finished)
 	local ges = #minetest.get_connected_players()
-	local form_n
+	local player_in_bed = get_player_in_bed_count()
 	local is_majority = (ges / 2) < player_in_bed
 
+	local form_n
+	local esc = minetest.formspec_escape
 	if finished then
-		form_n = beds.formspec .. "label[2.7,11; Good morning.]"
+		form_n = beds.formspec .. "label[2.7,9;" .. esc(S("Good morning.")) .. "]"
 	else
-		form_n = beds.formspec .. "label[2.2,11;" .. tostring(player_in_bed) ..
-			" of " .. tostring(ges) .. " players are in bed]"
+		form_n = beds.formspec .. "label[2.2,9;" ..
+			esc(S("@1 of @2 players are in bed", player_in_bed, ges)) .. "]"
 		if is_majority and is_night_skip_enabled() then
-			form_n = form_n .. "button_exit[2,8;4,0.75;force;Force night skip]"
+			form_n = form_n .. "button_exit[2,6;4,0.75;force;" ..
+				esc(S("Force night skip")) .. "]"
 		end
 	end
 
@@ -132,16 +181,36 @@ function beds.skip_night()
 	minetest.set_timeofday(0.23)
 end
 
+local update_scheduled = false
+local function schedule_update()
+	if update_scheduled then
+		-- there already is an update scheduled; don't schedule more to prevent races
+		return
+	end
+	update_scheduled = true
+	minetest.after(2, function()
+		update_scheduled = false
+		if not is_sp then
+			update_formspecs(is_night_skip_enabled())
+		end
+		if is_night_skip_enabled() then
+			-- skip the night and let all players stand up
+			beds.skip_night()
+			beds.kick_players()
+		end
+	end)
+end
+
 function beds.on_rightclick(pos, player)
 	local name = player:get_player_name()
-	local ppos = player:getpos()
+	local ppos = player:get_pos()
 	local tod = minetest.get_timeofday()
 
-	if tod > 0.2 and tod < 0.805 then
+	if tod > beds.day_interval.start and tod < beds.day_interval.finish then
 		if beds.player[name] then
 			lay_down(player, nil, nil, false)
 		end
-		minetest.chat_send_player(name, "You can only sleep at night.")
+		minetest.chat_send_player(name, S("You can only sleep at night."))
 		return
 	end
 
@@ -157,20 +226,20 @@ function beds.on_rightclick(pos, player)
 		update_formspecs(false)
 	end
 
-	-- skip the night and let all players stand up
 	if check_in_beds() then
-		minetest.after(2, function()
-			if not is_sp then
-				update_formspecs(is_night_skip_enabled())
-			end
-			if is_night_skip_enabled() then
-				beds.skip_night()
-				beds.kick_players()
-			end
-		end)
+		schedule_update()
 	end
 end
 
+function beds.can_dig(bed_pos)
+	-- Check all players in bed which one is at the expected position
+	for _, player_bed_pos in pairs(beds.bed_position) do
+		if vector.equals(bed_pos, player_bed_pos) then
+			return false
+		end
+	end
+	return true
+end
 
 -- Callbacks
 -- Only register respawn callback if respawn enabled
@@ -180,7 +249,7 @@ if enable_respawn then
 		local name = player:get_player_name()
 		local pos = beds.spawn[name]
 		if pos then
-			player:setpos(pos)
+			player:set_pos(pos)
 			return true
 		end
 	end)
@@ -191,13 +260,20 @@ minetest.register_on_leaveplayer(function(player)
 	lay_down(player, nil, nil, false, true)
 	beds.player[name] = nil
 	if check_in_beds() then
-		minetest.after(2, function()
-			update_formspecs(is_night_skip_enabled())
-			if is_night_skip_enabled() then
-				beds.skip_night()
-				beds.kick_players()
-			end
-		end)
+		schedule_update()
+	end
+end)
+
+minetest.register_on_dieplayer(function(player)
+	local name = player:get_player_name()
+	local in_bed = beds.player
+	local pos = player:get_pos()
+	local yaw = get_look_yaw(pos)
+
+	if in_bed[name] then
+		lay_down(player, nil, pos, false)
+		player:set_look_horizontal(yaw)
+		player:set_pos(pos)
 	end
 end)
 
@@ -205,16 +281,25 @@ minetest.register_on_player_receive_fields(function(player, formname, fields)
 	if formname ~= "beds_form" then
 		return
 	end
+
+	-- Because "Force night skip" button is a button_exit, it will set fields.quit
+	-- and lay_down call will change value of player_in_bed, so it must be taken
+	-- earlier.
+	local last_player_in_bed = get_player_in_bed_count()
+
 	if fields.quit or fields.leave then
 		lay_down(player, nil, nil, false)
 		update_formspecs(false)
 	end
 
 	if fields.force then
-		update_formspecs(is_night_skip_enabled())
-		if is_night_skip_enabled() then
+		local is_majority = (#minetest.get_connected_players() / 2) < last_player_in_bed
+		if is_majority and is_night_skip_enabled() then
+			update_formspecs(true)
 			beds.skip_night()
 			beds.kick_players()
+		else
+			update_formspecs(false)
 		end
 	end
 end)
