@@ -715,6 +715,54 @@ function default.register_craft_metadata_copy(ingredient, result)
 	end)
 end
 
+--
+-- Log API / helpers
+--
+
+local log_non_player_actions = minetest.settings:get_bool("log_non_player_actions", false)
+
+local is_pos = function(v)
+	return type(v) == "table" and
+		type(v.x) == "number" and type(v.y) == "number" and type(v.z) == "number"
+end
+
+function default.log_player_action(player, ...)
+	local msg = player:get_player_name()
+	if player.is_fake_player or not player:is_player() then
+		if not log_non_player_actions then
+			return
+		end
+		msg = msg .. "(" .. (type(player.is_fake_player) == "string"
+			and player.is_fake_player or "*") .. ")"
+	end
+	for _, v in ipairs({...}) do
+		-- translate pos
+		local part = is_pos(v) and minetest.pos_to_string(v) or v
+		-- no leading spaces before punctuation marks
+		msg = msg .. (string.match(part, "^[;,.]") and "" or " ") .. part
+	end
+	minetest.log("action",  msg)
+end
+
+local nop = function() end
+function default.set_inventory_action_loggers(def, name)
+	local on_move = def.on_metadata_inventory_move or nop
+	def.on_metadata_inventory_move = function(pos, from_list, from_index,
+			to_list, to_index, count, player)
+		default.log_player_action(player, "moves stuff in", name, "at", pos)
+		return on_move(pos, from_list, from_index, to_list, to_index, count, player)
+	end
+	local on_put = def.on_metadata_inventory_put or nop
+	def.on_metadata_inventory_put = function(pos, listname, index, stack, player)
+		default.log_player_action(player, "moves", stack:get_name(), stack:get_count(), "to", name, "at", pos)
+		return on_put(pos, listname, index, stack, player)
+	end
+	local on_take = def.on_metadata_inventory_take or nop
+	def.on_metadata_inventory_take = function(pos, listname, index, stack, player)
+		default.log_player_action(player, "takes", stack:get_name(), stack:get_count(), "from", name, "at", pos)
+		return on_take(pos, listname, index, stack, player)
+	end
+end
 
 --
 -- NOTICE: This method is not an official part of the API yet.
